/* $Header: cmd.h,v 3.0 98/06/09 17:01:46 carl Exp $ */

#ifndef CMDLIST_H_INCLUDED
#define CMDLIST_H_INCLUDED

#include <stdio.h>
#include "userfun.h"

#define CMDSTACK_SIZE     200
#define CMD_BELL_TAB       60  /* the length of a line for printing the cmdlist
				  (Like the bell for a typewriter) */

enum cmd_vars {
  CMDVAR_X,
  CMDVAR_Y,
  CMDVAR_Z,
  CMDVAR_U,
  CMDVAR_T,
  CMDVAR_I,
  CMDVAR_XLBD,
  CMDVAR_XUBD,
  CMDVAR_YLBD,
  CMDVAR_YUBD,
  CMDVAR_ALPHA,
  CMDVAR_BETA,
  CMDVAR_TAB_SIZE
};

enum cmd_codes {
  CMD_NULL,
  CMD_XVAR,
  CMD_YVAR,
  CMD_ZVAR,
  CMD_UVAR,
  CMD_TVAR,
  CMD_CONSTANT,
  CMD_PLUS,
  CMD_MINUS,
  CMD_MUL,
  CMD_DIV,
  CMD_POW,
  CMD_UMINUS,
  CMD_SQRT,
  CMD_SIN,
  CMD_COS,
  CMD_TAN,
  CMD_ASIN,
  CMD_ACOS,
  CMD_ATAN,
  CMD_ACOT,
  CMD_SINH,
  CMD_COSH,
  CMD_TANH,
  CMD_SECH,
  CMD_EXP,
  CMD_LOG,
  CMD_LOG10,
  CMD_MYLOG,
  CMD_XLBD,
  CMD_XUBD,
  CMD_YLBD,
  CMD_YUBD,
  CMD_ALPHA,
  CMD_BETA,
  CMD_FLOOR,
  CMD_CEIL,
  CMD_ABS,
  CMD_BINMIN,
  CMD_BINMAX,
  CMD_ISLT,
  CMD_ISLE,
  CMD_ISEQ,
  CMD_ISGT,
  CMD_ISGE,
  CMD_BOUND_LOW,
  CMD_BOUND_UPP,
  CMD_MIN,
  CMD_MAX,
  CMD_MINTEST,
  CMD_MAXTEST,
  CMD_INTERVAL,
  CMD_TEST,
  CMD_USERFUN,
  CMD_CODE_SIZE
};

typedef struct cmdvar {
  int    size;
  double *val;
} cmdvar_struct;

typedef struct cmdvarname {
  int  size;
  char **name;
} cmdvarname_struct;

typedef struct cmd cmdlist, *cmdptr;
struct cmd {
  int opcode;
  union {
    int	    index;
    double  val;
    USRFNTYPE usrfn;
    cmdlist *cPtr;
    cmdlist **incmd; /* internal command lists for the conditional expressions */
  } operand;
  int ndisc;  /* number of tests for internal commands */
  int active; /* current active branch for internal commands */
};

#if defined(__STDC__) || defined (__cplusplus)
typedef void (*cmdlist_fun)(cmdptr cmd);
#else
typedef void (*cmdlist_fun)();
#endif

#if defined(__STDC__) || defined(__cplusplus)
# define P_(s) s
#else
# define P_(s) ()
#endif

void cmdvar_init P_(());

void cmdvar_load P_((int num, int nval, double *val));

void cmdvarname_load P_((int num, int nval, char **name));

void cmdvar_unload P_((int var));

void cmdvarname_unload P_((int var));

double cmdvar_get_value P_((int var, int index));

cmdlist	*cmdlist_copy P_((cmdlist *cmd));

double new_cmdlist_run P_((cmdlist *cmdptr));

double cmdlist_run P_((cmdlist *cmdptr));

int cmdlist_active P_((cmdlist *cmd));

void cmdlist_print P_((cmdlist *cmdarray, FILE *fp, int *printed));

void cmdlist_cprint P_((cmdlist *cmdarray, FILE *fp, int *printed));

void cmdlist_free P_((cmdlist *cmd));

void get_children P_((int curcmd, int *cmdind, cmdlist *cmd, int **child));

void print_children P_((int curcmd, cmdlist *cmd, int **child, FILE *fp, int *ncp));

void cprint_children P_((int curcmd, cmdlist *cmd, int **child, FILE *fp, int *ncp));

void cmdlist_xvar P_((cmdptr cmd));
void cmdlist_yvar P_((cmdptr cmd));
void cmdlist_zvar P_((cmdptr cmd));
void cmdlist_uvar P_((cmdptr cmd));
void cmdlist_tvar P_((cmdptr cmd));
void cmdlist_constant P_((cmdptr cmd));
void cmdlist_plus P_((cmdptr cmd));
void cmdlist_minus P_((cmdptr cmd));
void cmdlist_mul P_((cmdptr cmd));
void cmdlist_div P_((cmdptr cmd));
void cmdlist_pow P_((cmdptr cmd));
void cmdlist_uminus P_((cmdptr cmd));
void cmdlist_sin P_((cmdptr cmd));
void cmdlist_cos P_((cmdptr cmd));
void cmdlist_exp P_((cmdptr cmd));
void cmdlist_log P_((cmdptr cmd));
void cmdlist_sqrt P_((cmdptr cmd));
void cmdlist_min P_((cmdptr cmd));
void cmdlist_max P_((cmdptr cmd));
void cmdlist_mintest P_((cmdptr cmd));
void cmdlist_maxtest P_((cmdptr cmd));
void cmdlist_interval P_((cmdptr cmd));

#ifdef INTERVAL_MATH
double interval_low P_((cmdlist *cmdPtr));
double interval_upp P_((cmdlist *cmdPtr));
#endif

#undef P_

#endif /* CMD_H_INCLUDED */
